package checkers.nullness.quals;

import java.lang.annotation.*;

import checkers.nullness.NullnessChecker;
import checkers.quals.SubtypeOf;
import checkers.quals.TypeQualifier;

/**
 * Indicates that a field is lazily initialized.  Lazily initialized fields
 * have these two property:
 * <ol>
 * <li id="1">The field may not be initialized in the constructor</li>
 * <li id="2">A variable may only be assigned to other nonnull references.
 *   An implication of this premise is that once the field is initialized,
 *   it may never be {@code null} again.</li>
 * </ol>
 *
 * <p>
 *
 * This annotation is associated with the {@link NullnessChecker}.
 *
 * @see Nullable
 * @see NullnessChecker
 * @manual #nullness-checker Nullness Checker
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.FIELD)
@TypeQualifier
@SubtypeOf( Nullable.class )
public @interface LazyNonNull {

}
